@@ -1,4 +1,4 @@
-# Copyright 2001-7 by Roger Bivand 
+# Copyright 2001-2010 by Roger Bivand 
 #
 
 lm.morantest <- function(model, listw, zero.policy=NULL, 
@@ -17,7 +17,8 @@ lm.morantest <- function(model, listw, zero.policy=NULL,
 	    subset <- !(1:length(listw$neighbours) %in% na.act)
 	    listw <- subset(listw, subset, zero.policy=zero.policy)
 	}
- 	N <- length(listw$neighbours)
+# 101124 Aleksandr Andreev
+ 	N <- as.double(length(listw$neighbours))
 	u <- resfun(model)
 	if (N != length(u)) 
             stop("objects of different length")
@@ -31,7 +32,9 @@ lm.morantest <- function(model, listw, zero.policy=NULL,
 	S0 <- sum(unlist(listw.U$weights))
 	S1 <- 0.5 * sum((2*unlist(listw.U$weights))^2)
 	lu <- lag.listw(listw.U, u, zero.policy=zero.policy)
-	if (zero.policy) N <- length(which(card(listw$neighbours) > 0))
+# 101125 Aleksandr Andreev
+	if (zero.policy)
+            N <- as.double(length(which(card(listw$neighbours) > 0)))
 	I <- (N/S0) * ((t(u) %*% lu) / (t(u) %*% u))
 	p <- model$rank
 	p1 <- 1:p
